#
# spec file for package ghc-lens
#
# Copyright (c) 2021 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global pkg_name lens
%bcond_with tests
Name:           ghc-%{pkg_name}
Version:        5.1
Release:        bp154.1.15
Summary:        Lenses, Folds and Traversals
License:        BSD-2-Clause
URL:            https://hackage.haskell.org/package/%{pkg_name}
Source0:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/%{pkg_name}-%{version}.tar.gz
Source1:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/revision/1.cabal#/%{pkg_name}.cabal
BuildRequires:  ghc-Cabal-devel
BuildRequires:  ghc-array-devel
BuildRequires:  ghc-assoc-devel
BuildRequires:  ghc-base-orphans-devel
BuildRequires:  ghc-bifunctors-devel
BuildRequires:  ghc-bytestring-devel
BuildRequires:  ghc-call-stack-devel
BuildRequires:  ghc-comonad-devel
BuildRequires:  ghc-containers-devel
BuildRequires:  ghc-contravariant-devel
BuildRequires:  ghc-distributive-devel
BuildRequires:  ghc-exceptions-devel
BuildRequires:  ghc-filepath-devel
BuildRequires:  ghc-free-devel
BuildRequires:  ghc-hashable-devel
BuildRequires:  ghc-indexed-traversable-devel
BuildRequires:  ghc-indexed-traversable-instances-devel
BuildRequires:  ghc-kan-extensions-devel
BuildRequires:  ghc-mtl-devel
BuildRequires:  ghc-parallel-devel
BuildRequires:  ghc-profunctors-devel
BuildRequires:  ghc-reflection-devel
BuildRequires:  ghc-rpm-macros
BuildRequires:  ghc-semigroupoids-devel
BuildRequires:  ghc-strict-devel
BuildRequires:  ghc-tagged-devel
BuildRequires:  ghc-template-haskell-devel
BuildRequires:  ghc-text-devel
BuildRequires:  ghc-th-abstraction-devel
BuildRequires:  ghc-these-devel
BuildRequires:  ghc-transformers-compat-devel
BuildRequires:  ghc-transformers-devel
BuildRequires:  ghc-unordered-containers-devel
BuildRequires:  ghc-vector-devel
ExcludeArch:    %{ix86}
%if %{with tests}
BuildRequires:  ghc-HUnit-devel
BuildRequires:  ghc-QuickCheck-devel
BuildRequires:  ghc-deepseq-devel
BuildRequires:  ghc-simple-reflect-devel
BuildRequires:  ghc-test-framework-devel
BuildRequires:  ghc-test-framework-hunit-devel
BuildRequires:  ghc-test-framework-quickcheck2-devel
%endif

%description
This package comes "Batteries Included" with many useful lenses for the types
commonly used from the Haskell Platform, and with tools for automatically
generating lenses and isomorphisms for user-supplied data types.

The combinators in 'Control.Lens' provide a highly generic toolbox for
composing families of getters, folds, isomorphisms, traversals, setters and
lenses and their indexed variants.

An overview, with a large number of examples can be found in the
<https://github.com/ekmett/lens#lens-lenses-folds-and-traversals README>.

An introductory video on the style of code used in this library by Simon Peyton
Jones is available from
<http://skillsmatter.com/podcast/scala/lenses-compositional-data-access-and-manipulation
Skills Matter>.

A video on how to use lenses and how they are constructed is available on
<http://youtu.be/cefnmjtAolY?hd=1 youtube>.

Slides for that second talk can be obtained from
<http://comonad.com/haskell/Lenses-Folds-and-Traversals-NYC.pdf comonad.com>.

More information on the care and feeding of lenses, including a brief tutorial
and motivation for their types can be found on the
<https://github.com/ekmett/lens/wiki lens wiki>.

A small game of 'pong' and other more complex examples that manage their state
using lenses can be found in the
<https://github.com/ekmett/lens/blob/master/examples/ example folder>.

/Lenses, Folds and Traversals/

With some signatures simplified, the core of the hierarchy of lens-like
constructions looks like:

<<http://i.imgur.com/ALlbPRa.png>>

<images/Hierarchy.png (Local Copy)>

You can compose any two elements of the hierarchy above using '(.)' from the
'Prelude', and you can use any element of the hierarchy as any type it linked
to above it.

The result is their lowest upper bound in the hierarchy (or an error if that
bound doesn't exist).

For instance:

* You can use any 'Traversal' as a 'Fold' or as a 'Setter'.

* The composition of a 'Traversal' and a 'Getter' yields a 'Fold'.

/Minimizing Dependencies/

If you want to provide lenses and traversals for your own types in your own
libraries, then you can do so without incurring a dependency on this (or any
other) lens package at all.

/e.g./ for a data type:

> data Foo a = Foo Int Int a

You can define lenses such as

> -- bar :: Lens' (Foo a) Int > bar :: Functor f => (Int -> f Int) -> Foo a ->
f (Foo a) > bar f (Foo a b c) = fmap (a' -> Foo a' b c) (f a)

> -- quux :: Lens (Foo a) (Foo b) a b > quux :: Functor f => (a -> f b) -> Foo
a -> f (Foo b) > quux f (Foo a b c) = fmap (Foo a b) (f c)

without the need to use any type that isn't already defined in the 'Prelude'.

And you can define a traversal of multiple fields with
'Control.Applicative.Applicative':

> -- traverseBarAndBaz :: Traversal' (Foo a) Int > traverseBarAndBaz ::
Applicative f => (Int -> f Int) -> Foo a -> f (Foo a) > traverseBarAndBaz f
(Foo a b c) = Foo <$> f a <*> f b <*> pure c

What is provided in this library is a number of stock lenses and traversals for
common haskell types, a wide array of combinators for working them, and more
exotic functionality, (/e.g./ getters, setters, indexed folds, isomorphisms).

%package devel
Summary:        Haskell %{pkg_name} library development files
Requires:       %{name} = %{version}-%{release}
Requires:       ghc-compiler = %{ghc_version}
Requires(post): ghc-compiler = %{ghc_version}
Requires(postun): ghc-compiler = %{ghc_version}

%description devel
This package provides the Haskell %{pkg_name} library development files.

%prep
%autosetup -n %{pkg_name}-%{version}
cp -p %{SOURCE1} %{pkg_name}.cabal

%build
%ghc_lib_build

%install
%ghc_lib_install

%check
%cabal_test

%post devel
%ghc_pkg_recache

%postun devel
%ghc_pkg_recache

%files -f %{name}.files
%license LICENSE

%files devel -f %{name}-devel.files
%doc AUTHORS.markdown CHANGELOG.markdown README.markdown examples

%changelog
* Sun Dec 26 2021 Peter Simons <psimons@suse.com>
- Update lens to version 5.1 revision 1.
  Upstream has revised the Cabal build instructions on Hackage.
* Tue Nov 16 2021 psimons@suse.com
- Update lens to version 5.1.
  5.1 [2021.11.15]
  - ---------------
  * Allow building with GHC 9.2.
  * Drop support for GHC 7.10 and older.
  * The type of `_ConP` in `Language.Haskell.TH.Lens` is now
    `Prism' Pat (Name, [Type], [Pat])` instead of `Prism' Pat (Name, [Pat])`
    when building with `template-haskell-2.18` or later.
  * Define `_CharTyLit` in `Language.Haskell.TH.Lens` when building with
    `template-haskell-2.18` or later.
  * Add `Prefixed` and `Suffixed` classes to `Control.Lens.Prism`, which provide
    `prefixed` and `suffixed` prisms for prefixes and suffixes of sequence types.
    These classes generalize the `prefixed` and `suffixed` functions in
    `Data.List.Lens`, which were previously top-level functions. In addition to
    providing `Prefixed` and `Suffixed` instances for lists, instances for `Text`
    and `ByteString` types are also provided.
    At present, `Prefixed` and `Suffixed` are re-exported from `Data.List.Lens`
    for backwards compatibility. This may change in a future version of `lens`,
    however.
  * Add a `traversal` function to `Control.Lens.Traversal`. This function, aside
    from acting as a `Traversal` counterpart to the `lens` and `prism` functions,
    provides documentation on how to define `Traversal`s.
  * Add a `matching'` function to `Control.Lens.Prism`. `matching'` is like
    `matching`, but with a slightly more general type signature that allows it to
    work with combinations of `Lens`es, `Prism`s, and `Traversal`s.
* Thu Sep  2 2021 psimons@suse.com
- Update lens to version 5.0.1 revision 2.
  Upstream has revised the Cabal build instructions on Hackage.
* Mon May 17 2021 psimons@suse.com
- Update lens to version 5.0.1 revision 1.
  Upstream has revised the Cabal build instructions on Hackage.
* Wed Feb 24 2021 psimons@suse.com
- Update lens to version 5.0.1.
  5.0.1 [2021.02.24]
  - -----------------
  * Fix a bug in which `makeLenses` could produce ill kinded optics for
    poly-kinded datatypes in certain situations.
* Fri Feb 19 2021 psimons@suse.com
- Update lens to version 5.
  5 [2021.02.17]
  - -------------
  * Support building with GHC 9.0.
  * Remove the `Swapped` type class in favor of `Swap` from the `assoc` package.
  * Remove the `Strict` type class in favor of `Strict` from the `strict` package.
    The `swapped`, `strict` and `lazy` isomorphisms are now defined using "new" type classes.
    Users which define own instances of old type classes are advised to
    define instances of the new ones.
    ```haskell
    import qualified Data.Bifunctor.Swap as Swap
    import qualified Control.Lens        as Lens
    instance Swap.Swap MyType where
    swap = ...
    [#]if !MIN_VERSION_lens(4,20,0)
    instance Lens.Swapped MyType where
    swapped = iso Swap.swap Swap.swap
    [#]endif
    ```
  * The `FunctorWithIndex`, `FoldableWithIndex` and `TraversableWithIndex` type classes
    have been migrated to a new package,
    [`indexed-traversable`](https://hackage.haskell.org/package/indexed-traversable).
    The `imapped`, `ifolded` and `itraversed` methods are now top-level functions.
    If you are not defining these methods in your instances,
    you don't need to change your definitions.
    Beware: the `optics-core` package (versions <0.4) defines similar classes,
    and will also migrate to use `indexed-traversable` classes. Therefore, you
    might get duplicate instance errors if your package defines both.
    If you define your own `FunctorWithIndex` etc. instances,
    we recommend that you depend directly on the `indexed-traversable` package.
    If you want to continue support `lens-4` users, you may write
    ```haskell
  - - from indexed-traversable
    import Data.Functor.WithIndex
  - - from lens
    import qualified Control.Lens as L
  - - your (indexed) container
    data MySeq a = ...
  - - indexed-traversable instance
    instance FunctorWithIndex     Int MySeq where imap = ...
    instance FoldableWithIndex    Int MySeq where ifoldMap = ...
    instance TraversableWithIndex Int MySeq where itraverse = ...
  - - lens <5 instance, note the !
    [#]if !MIN_VERSION_lens(5,0,0)
    instance L.FunctorWithIndex     Int MySeq where imap = imap
    instance L.FoldableWithIndex    Int MySeq where ifoldMap = ifoldMap
    instance L.TraversableWithIndex Int MySeq where itraverse = itraverse
    [#]endif
    ```
    In other words, always provide `indexed-traversable` instances.
    If your package depends on `lens` and allows `lens-4`,
    you should additionally provide instances for `lens-4` type classes
    that can reuse the `indexed-traversable` instances.
  * Make the functions in `Control.Lens.TH` work more robustly with poly-kinded
    data types. This can cause a breaking change under certain situations:
  * TH-generated optics for poly-kinded data types are now much more likely to
    mention kind variables in their definitions, which will require enabling
    the `PolyKinds` extension at use sites in order to typecheck.
  * Because TH-generated optics now quantify more kind variables than they did
    previously, this can affect the order of visible type applications.
  * Generalize the types of `generic` and `generic1` to allow type-changing
    updates. If you wish to use the old, more restricted types of these
    functions, use `simple . generic` or `simple . generic1` instead.
  * Add `Control.Lens.Profunctor` with conversion functions to and from
    profunctor optic representation.
  * Add `Control.Lens.Review.reviewing`, which is like `review` but with a more
    polymorphic type.
  * Mark `Control.Lens.Equality` as Trustworthy.
  * The build-type has been changed from `Custom` to `Simple`.
    To achieve this, the `doctests` test suite has been removed in favor of using [`cabal-docspec`](https://github.com/phadej/cabal-extras/tree/master/cabal-docspec) to run the doctests.
  * Use `alterF` in `At (HashMap k)` instance implementation.
  * Use `alterF` in `At` and `Contains` instances for `Set`, `IntSet`, and
    `HashSet`.
  * Avoid re-inserting keys already present in `ix` for `Set`, `IntSet`,
    and `HashSet`. For `Set` and `HashSet`, this changes the semantics
    slightly; if the user-supplied key is `==` to one already present in
    the set, then the latter will not be replaced in the result.
  * Consume `()` values lazily in `Control.Lens.At`.
* Mon Feb  8 2021 psimons@suse.com
- Update lens to version 4.19.2 revision 5.
  Upstream has revised the Cabal build instructions on Hackage.
* Sun Jan 31 2021 psimons@suse.com
- Update lens to version 4.19.2 revision 4.
  Upstream has revised the Cabal build instructions on Hackage.
* Wed Jan 20 2021 psimons@suse.com
- Update lens to version 4.19.2 revision 3.
  Upstream has revised the Cabal build instructions on Hackage.
* Thu Dec 17 2020 Ondřej Súkup <mimi.vx@gmail.com>
- disable %%{ix86} build
* Tue Oct  6 2020 psimons@suse.com
- Update lens to version 4.19.2 revision 2.
  Upstream has revised the Cabal build instructions on Hackage.
* Mon Aug 31 2020 psimons@suse.com
- Update lens to version 4.19.2 revision 1.
  Upstream has revised the Cabal build instructions on Hackage.
* Tue Aug 18 2020 Peter Simons <psimons@suse.com>
- Replace %%setup -q with the more modern %%autosetup macro.
* Tue Jun 16 2020 Peter Simons <psimons@suse.com>
- Re-generate file with latest version of spec-cleaner.
* Wed May  6 2020 psimons@suse.com
- Update lens to version 4.19.2.
  4.19.2 [2020.04.15]
  - ------------------
  * Remove the test suite's dependency on `test-framework-th`.
* Thu Feb 27 2020 psimons@suse.com
- Update lens to version 4.19.1.
  4.19.1 [2020.02.13]
  - ------------------
  * Fix a bug introduced in 4.19 where using `_TupE` to `preview` a value would
    always fail.
* Fri Feb  7 2020 psimons@suse.com
- Update lens to version 4.19.
  4.19 [2020.02.03]
  - ----------------
  * Support building with GHC 8.10.
  * The types of `_TupE` and `_UnboxedTupE` are now `Prism' Exp [Maybe Exp]`
    when built against `template-haskell-2.16` or later to reflect the new
    types of `TupE` and `UnboxedTupE`.
  * Add `_ForallVisT` and `_BytesPrimL` prisms when building against
    `template-haskell-2.16` or later.
  * Make `<>~` and `<>=` and their `<op` and `<<op` state variants require only
    `Semigroup`, not `Monoid`.
  * Add `{Functor,Foldable,Traversable}WithIndex` instances for
    `Control.Applicative.Const` and `Data.Functor.Constant.Constant`.
* Fri Nov  8 2019 Peter Simons <psimons@suse.com>
- Drop obsolete group attributes.
* Sat Sep 14 2019 psimons@suse.com
- Update lens to version 4.18.1.
  4.18.1 [2019.09.13]
  - ------------------
  * Remove the use of `cpp-options: -traditional`. This should be unnecessary
    on all versions of GHC that `lens` supports, as modern GHCs already use
    `-traditional` internally during preprocessing. More critically, the use
    of `cpp-options: -traditional` breaks profiling builds on GHC 8.8
    (see https://gitlab.haskell.org/ghc/ghc/issues/17185).
* Sat Sep  7 2019 psimons@suse.com
- Update lens to version 4.18.
  Upstream has edited the change log file since the last release in
  a non-trivial way, i.e. they did more than just add a new entry
  at the top. You can review the file at:
  http://hackage.haskell.org/package/lens-4.18/src/CHANGELOG.markdown
* Thu Aug  8 2019 Peter Simons <psimons@suse.com>
- Update Cabal file to allow building with call-stack 0.2.x.
* Tue Jun 11 2019 Peter Simons <psimons@suse.com>
- Update Cabal file for more accurate build dependencies.
* Tue Apr 30 2019 psimons@suse.com
- Update lens to version 4.17.1.
  4.17.1 [2019.04.26]
  - ------------------
  * Support `th-abstraction-0.3.0.0` or later.
  * Only incur `semigroups` and `void` dependencies on old GHCs.
  * Add `holes1Of`.
  * Add `locally` (https://github.com/ekmett/lens/pull/829).
  * Add `ilocally` (https://github.com/ekmett/lens/pull/836).
  * Add a third `Prism` law.
  * Add `gplate1`.
  * Add `Wrapped`/`Rewrapped` instances for `Data.Monoid.Ap`.
* Thu Aug  2 2018 psimons@suse.com
- Add lens at version 4.17.
